<?php
	require_once('../pear/DB.php');
	include_once('../settings/settings.php');
	include_once('../include/globals.php');
	include_once('../include/loader.php');
	include_once('../include/functions.php');
	include_once('../include/page_specific/render_survey.php');

	// used to identify if the user supplied data is fully valid
	$is_valid = 1; 
	// for error tracking
	$error_log = Array();

	// make sure survey_id has been passed in and is valid
	if(!(isset($_POST['esy_survey_id']) && is_numeric($_POST['esy_survey_id']) && $_POST['esy_survey_id'] > 0)){ set_invalid_survey("survey_id is not in proper format"); }

	// make sure the host_url has been passed in and is valid
	if(!(isset($_POST['esy_host_url']) && strlen(trim($_POST['esy_host_url'])) > 0)){ set_invalid_survey("no host_url found"); }

	if($is_valid){
		$survey_id = $_POST['esy_survey_id'];

		// see if this survey is valid and running
		if(verify_running_survey($survey_id)){
			// fetch the question data from the DB to compare against the data passed in via post
			$questions_array = get_question_data_struct($survey_id);

			// set up php variables for any non-existing answer data so we can avoid the isset calls
			foreach($questions_array as $i => $question_struct){
				// place empty arrays where arrays are expected
				if($question_struct['question_type_id'] == $TID_CHECK_BOXES || $question_struct['question_type_id'] == $TID_SELECT_BOX){
					if(!isset($_POST["esy_q${i}_ans"])){ $_POST["esy_q${i}_ans"] = ${"esy_q${i}_ans"} = Array(); }
				}
				// otherwise, we're probably dealing with a string answer
				else{
					// if the string already exists, then trim it
					if(isset($_POST["esy_q${i}_ans"]) && !is_array($_POST["esy_q${i}_ans"])){ $_POST["esy_q${i}_ans"] = ${"esy_q${i}_ans"} = trim($_POST["esy_q${i}_ans"]); }
					// otherwise, force it to the empty string
					else{ $_POST["esy_q${i}_ans"] = ${"esy_q${i}_ans"} = ''; }
				} // end else
			} // end foreach

			// make sure the data supplied for each question is in the correct format
			foreach($questions_array as $i => $question_struct){
				// construct an array of valid option_ids
				$option_array = Array();
				foreach($question_struct['options'] as $idx => $option_struct){ $option_array[] = $option_struct['option_id']; }

				// if we're expecting an array, make sure we've got one
				if($question_struct['question_type_id'] == $TID_CHECK_BOXES || $question_struct['question_type_id'] == $TID_SELECT_BOX){
					// if the array doesn't exist, then the data isn't valid
					if(!is_array($_POST["esy_q${i}_ans"])){ set_invalid_survey("Ans${i} didn't find Array() answer."); }
					else{
						foreach($_POST["esy_q${i}_ans"] as $idx => $answer){
							// each entry in the array must be numeric and > 0
							if(!(is_numeric($answer) && $answer > 0)){ set_invalid_survey("Ans${i} Opt${idx} isn't a valid formatted option_id."); }
							else{
								// each option_id must also appear as a value in the DB
								if(!in_array($answer, $option_array)){ set_invalid_survey("Ans${i} Opt${idx} option_id not found in DB."); }
							}
						} // end foreach
					} // end else
				} // end if
				else{
					// for all other types of answers, we don't want an array
					if(is_array($_POST["esy_q${i}_ans"])){ set_invalid_survey("Ans${i} found unexpected Array() answer"); }
					else{
						// check for option id's set for radio and drop down questions
						if($question_struct['question_type_id'] == $TID_RADIO_BUTTONS || $question_struct['question_type_id'] == $TID_DROP_DOWN){
							// if an answer was supplied
							if(strlen($_POST["esy_q${i}_ans"]) > 0){
								// make sure the answer is numeric and > 0
								if(!(is_numeric($_POST["esy_q${i}_ans"]) && $_POST["esy_q${i}_ans"] > 0)){ set_invalid_survey("Ans${i} isn't a valid formatted option_id."); }
								else{
									// each option_id must also appear as a value in the DB
									if(!in_array($_POST["esy_q${i}_ans"], $option_array)){ set_invalid_survey("Ans${i} option_id not found in DB."); }
								} 
							} // end if
						} // end if
					} // end else
				} // end else
			} // end foreach

			if($is_valid){
				// make sure answers have been supplied for all required questions
				foreach($questions_array as $i => $question_struct){
					if($question_struct['is_required'] == 1){
						// arrays must have entries
						if(is_array($_POST["esy_q${i}_ans"]) && count($_POST["esy_q${i}_ans"]) == 0){ set_invalid_survey("Ans${i} no Array() answer found for required question."); }
						// strings must be non-blank
						if(!is_array($_POST["esy_q${i}_ans"]) && strlen($_POST["esy_q${i}_ans"]) == 0){ set_invalid_survey("Ans${i} no answer found for required question."); }
					}
				}
			} // end if
		} // end if
		else{ set_invalid_survey("survey either not found or not currently running."); }
	} // end if

	// if we've made it this far and through the gauntlet of data checking, then
	// we're ready to insert our data into the DB
	if($is_valid){
		$response_timestamp = time();

		// generate a unique hash id based on microtime to identify the response
		list($usec, $sec) = explode(' ', microtime());
		$unique_hash = md5((float) $sec . ((float) $usec * 1000000));

		// insert a response entry into the database
		$query = "INSERT INTO esy_responses (unique_hash, survey_id, response_timestamp, rec_status_id) VALUES ('${unique_hash}', ${esy_survey_id}, ${response_timestamp}, 1)";
		$db_result = $DBH->query($query);
		if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }

		// get the response id
		$query = "SELECT MAX(response_id) AS response_id FROM esy_responses WHERE unique_hash = '${unique_hash}' AND survey_id = $esy_survey_id AND response_timestamp = $response_timestamp";
		$db_result = $DBH->query($query);
		if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
		$db_row = $db_result->fetchRow();
		$response_id = $db_row['response_id'];

		// loop through all of the supplied survey answers and insert them into the DB
		foreach($questions_array as $i => $question_struct){
			$question_id = $question_struct['question_id'];
			$question_type = $question_struct['question_type_id'];
			$question_ans = ${"esy_q${i}_ans"};

			$query_array = Array();
			switch($question_type){
				// for text and textarea fields
				case $TID_TEXT_BOX: case $TID_TEXT_AREA:
					// if question was supplied, create insertion query
					if(strlen($question_ans) > 0){
						$question_ans = to_db_str($question_ans);
						$query_array[] = "INSERT INTO esy_text_answers (survey_id, question_id, response_id, answer_text, rec_status_id) VALUES (${esy_survey_id},${question_id},${response_id},'${question_ans}',1);";
					}
				break;
				// for radio buttons and drop downs
				case $TID_RADIO_BUTTONS: case $TID_DROP_DOWN:
					// if question was supplied, create insertion query
					if(strlen($question_ans) > 0){
						$query_array[] = "INSERT INTO esy_answers (survey_id, question_id, option_id, response_id, rec_status_id) VALUES (${esy_survey_id},${question_id},${question_ans},${response_id},1);";
					}
				break;
				// for checkboxes and select boxes
				case $TID_CHECK_BOXES: case $TID_SELECT_BOX:
					foreach($question_ans as $idx => $option_id){
						$query_array[] = "INSERT INTO esy_answers (survey_id, question_id, option_id, response_id, rec_status_id) VALUES (${esy_survey_id},${question_id},${option_id},${response_id},1);";
					}
				break;
			} // end switch
			foreach($query_array as $idx => $query){
				$db_result = $DBH->query($query);
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
			} // end foreach
		} // end for
	} // end if

	// if no errors were generated (valid survey data)
	if(count($error_log) == 0){ 
		// get the client submit count data struct from the cookie
		$submit_count_struct = get_submit_count_struct();

		// update the submit count for this survey
		if(isset($submit_count_struct[$survey_id])){ $submit_count_struct[$survey_id]++; }
		else{ $submit_count_struct[$survey_id] = 1; }

		// store the new survey submit count data to the cookie
		set_submit_count_cookie($submit_count_struct);

		// see if the host url has a query string
		$seperator = (strrpos("?", $_POST['esy_host_url']) === FALSE) ? "?" : "&";
		// construct the redirection url
		$redir_url = "${esy_host_url}${seperator}esy_thank_you=1";

		// redirect and exit
		echo '<html><head><META HTTP-EQUIV=Refresh CONTENT="0; URL='.$redir_url.'"></head></html>';
		exit(0);
	}
	// otherwise, display errors
	else{
		echo "<pre>\n";
		print_r($error_log); 
		/*
		echo "is_valid = $is_valid<br>\n";
		ksort($_POST);
		print_r($_POST);
		*/
		echo "</pre>\n";
	}

	// ---------------------------------------------------------------------------
	// track errors found in survey data along with descriptive message
	function set_invalid_survey($msg){
		global $is_valid, $error_log;
		$is_valid = 0;
		$error_log[] = $msg;
	}
	// ---------------------------------------------------------------------------
?>
