<?php
	require_once('../pear/DB.php');
	include_once('../settings/settings.php');
	include_once('../include/globals.php');
	include_once('../include/loader.php');
	include_once('../include/functions.php');
	include_once('../include/page_specific/render_survey.php');

	// make sure the survey_id was passed in on the hook
	$survey_id = isset($_GET['survey_id']) ? $_GET['survey_id'] : '';
	$is_preview = isset($_GET['is_preview']) ? $_GET['is_preview'] : 0;

	$survey_html = '';
	$thank_you_html = '';

	// we should only render a survey under the following 2 conditions \
	// (1) We're in preview mode and the survey_id exists in the database
	// (2) We're not in preview mode and the survey is non-draft and running
	if(($is_preview == 1 && get_survey_hash($survey_id) !== FALSE) || (!$is_preview && verify_running_survey($survey_id) == 1)){

		// get the current web path of this page
		$curr_page = 'render_survey.php';
		$curr_path = str_replace($curr_page,'','http://'.$_SERVER['SERVER_NAME'].$_SERVER['SCRIPT_NAME']);

		// fetch the survey data
		$survey_struct = get_survey_data_struct($survey_id);

		$display_survey = 1;
		// check the cookie data to see if the user has submitted to this survey before
		$submit_count_struct = get_submit_count_struct();

		// determine if the survey should be displayed based on the submit count
		// by the user and the max_submit_count set for the survey, and whether
		// it's to be previewed or not
		if(strlen($survey_struct['max_submit_count']) > 0 && $survey_struct['max_submit_count'] > 0){
			if(!$is_preview && is_array($submit_count_struct) && isset($submit_count_struct[$survey_id]) && $submit_count_struct[$survey_id] >= $survey_struct['max_submit_count']){
				$display_survey = 0;
			}
		}

		// if we're not going to display a survey, don't bother with creating it
		if($display_survey){
			// fetch all of our question data
			$questions_array = get_question_data_struct($survey_id);

			$survey_html .= "
					<link rel='stylesheet' type='text/css' href='${curr_path}survey_style.css'>
					<form name='esy_form' id='esy_form' method='POST' action='${curr_path}collect_stats.php' class='esy_form'>
					<input type='hidden' name='esy_survey_id' value='${survey_id}'>
					<input type='hidden' name='esy_host_url' id='esy_host_url' value=''>
					<script language='JavaScript' type='text/javascript'> 
					<!--
						".get_global_survey_js_code()."

						esy_set_host_url(); 
						".get_validation_js_code($questions_array)."
					//-->
					</script>

					<table cellspacing='0' cellpadding='0' border='0'>
					<tr>
						<td align='left' valign='top'>
							<div class='esy_instruction_text'>".$survey_struct['instructions']."<br></div>
							<span class='esy_required'>* </span><span class='esy_required_legend'>Indicates a Required Question<br><br></span>
						</td>
					</tr>
			";

			// loop through and render all of the questions
			foreach($questions_array as $question_num => $question_struct){

				$is_required = ($question_struct['is_required'] == 1) ? '*' : '';
				$survey_html .= "
					<tr>
						<td align='left' valign='top'><span class='esy_question_number'>".($question_num+1).") </span><span class='esy_question_text'><span class='esy_required'>$is_required</span>".$question_struct['question_text']."<br></span><span id='esy_q${question_num}_error' class='esy_error_text'></span></td>
					</tr>
					<tr>
						<td align='left' valign='top'>
							<div class='esy_answer_indent'>
				";

				switch($question_struct['question_type_id']){
					case 1: $survey_html .= get_text_field_html($question_num, $question_struct); break; 
					case 2: $survey_html .= get_textarea_html($question_num, $question_struct); break; 
					case 3: $survey_html .= get_radio_button_html($question_num, $question_struct); break;
					case 4: $survey_html .= get_check_box_html($question_num, $question_struct); break;
					case 5: $survey_html .= get_drop_down_html($question_num, $question_struct); break; 
					case 6: $survey_html .= get_select_box_html($question_num, $question_struct); break;
				} // end switch

				$survey_html .= "
							</div>
						</td>
					</tr>
					<tr><td><div class='esy_question_spacer'><br></div></td></tr>
				";

			} // end foreach

			$survey_html .= "
				<tr>
					<td align='left' valign='middle'>
			";
			if($is_preview){
				$survey_html .= "<input onclick='esy_preview_form(".count($questions_array).");' type='button' name='Submit' value='Submit' class='esy_form_button'>\n";
			}
			else{
				$survey_html .= "<input onclick='esy_submit_form(".count($questions_array).");' type='button' name='Submit' value='Submit' class='esy_form_button'>\n";
			}
			$survey_html .= "
						<input type='reset' name='' value='Reset' class='esy_form_button'>
					</td>
				</tr>
				</table>
				</form>
			";

		} // end if

		// prepare the html for displaying the thankyou message
		$thank_you_html .= "
			<link rel='stylesheet' type='text/css' href='${curr_path}survey_style.css'>
			<table cellspacing='0' cellpadding='0' border='0'>
			<tr>
				<td align='left' valign='top'><div class='esy_instruction_text'>".$survey_struct['confirmation']."<br></div></td>
			</tr>
			</table>
		";
	} // end if

	// as a very last step, we have to prepare the javascript code which is
	// returned. The logic is simple. If the document's URL contains
	// 'esy_thank_you=1', then we know we need to display the thank you message.
	// If not, we display the full survey to the user
	$js_code = "
		if(document.URL){
			// see if the hook contains info as to whether we need to display the thankyou message
			if(document.URL.indexOf('esy_thank_you=1') > -1){
				document.write(\"".to_js_str($thank_you_html)."\");
			}
			// otherwise, display the survey code
			else{
				document.write(\"".to_js_str($survey_html)."\");
			}
		}
	";
	echo $js_code;
?>
