<?php
	// --------------------------------------------------------------------------
	// This function takes a string and formats it so that it can be used in a DB query 
	// assignment. It is assumed that the string is "pure" and has had no slashes added.
	function to_db_str($str){
		if($str != ''){
			// turn ticks into two ticks ' -> ''
			$str = str_replace("'","''",$str); 
		}
		return $str;
	}
	// --------------------------------------------------------------------------
	// This function takes a string and formats it so that it can be used in a JavaScript 
	// assignment. It is assumed that the string is "pure" and has had no slashes added.
	function to_js_str($str){
		if($str != ''){
			// add slashes to ticks, quotes, backslashes, etc.
			$str = addslashes($str);
			// turn all newlines to \n form and 
			$str = str_replace("\r\n","\n",$str); 
			$str = str_replace("\r","\n",$str); 
			$str = str_replace("\n",'\n',$str); 
		}
		return $str;
	}
	// --------------------------------------------------------------------------
	// this function takes a string and prepares it so that it can be placed
	// inside an html tag attribute. This is important when loading a value into
	// a text input like <input type='text' value='$value_goes_here'> To avoid
	// breaking the html and/or submitting wrong data, we must prep the string
	function to_html_attrib_str($str){
		if($str != ''){
			$str = str_replace("&",'&amp;',$str);
			$str = str_replace('"','&#34;',$str);
			$str = str_replace("'",'&#39;',$str);
			$str = str_replace("<",'&lt;',$str);
			$str = str_replace(">",'&gt;',$str);
			$str = str_replace("\r\n",'&#13;',$str);
			$str = str_replace("\n",'&#13;',$str);
			$str = str_replace("\r",'&#13;',$str);
		}
		return $str;
	}
	// --------------------------------------------------------------------------
	// this function analyzes the server variables to figure out what the
	// current page's url is including any query string attached. This does not
	// return the protocol used (ie http).
	function get_curr_url(){
		$curr_url = $_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'];
		$curr_qs = (isset($_SERVER['QUERY_STRING'])) ? $_SERVER['QUERY_STRING'] : '';
		if(strlen($curr_qs) > 0){ $curr_url .= '?' . $curr_qs; }
		return $curr_url;
	}
	// --------------------------------------------------------------------------
	// This function takes a survey id and attempts to locate the survey hash
	// associasted with it. Returns the hash string on success, otherwise FALSE
	function get_survey_hash($survey_id){
		global $DBH;
		$ret = FALSE;
		if(strlen($survey_id) > 0 && is_numeric($survey_id)){
			$query = "SELECT survey_hash FROM esy_surveys WHERE survey_id = $survey_id AND rec_status_id = 1";
			$db_result = $DBH->query($query);
			if(!DB::isError($db_result) && $db_result->numRows() == 1){ 
				$db_row = $db_result->fetchRow();
				$ret = $db_row['survey_hash'];
			}
		}
		return $ret;
	}
	// --------------------------------------------------------------------------
	// This function is an attempt at a DB agnostic way to retrieve the column
	// id of the last record inserted into a table. It takes the column name,
	// database table, the column name that stores the data hash, and the actual
	// hash that was inserted. The has allows us to be very confident that we've
	// retrieved the correct id.
	function get_last_insert_id($id_col_name,$db_table_name,$hash_col_name,$hash){
		global $DBH;
		$id = FALSE;
		$query = "SELECT MAX($id_col_name) AS $id_col_name FROM $db_table_name WHERE $hash_col_name = '$hash' AND rec_status_id = 1;";
		$db_result = $DBH->query($query);
		if(!DB::isError($db_result)){
			$db_row = $db_result->fetchRow();
			$id = $db_row[$id_col_name];
		}
		return $id;
	}
	// --------------------------------------------------------------------------
	// This function returns the number of survey's with a draft status
	function get_survey_drafts_count(){
		global $DBH;
		$survey_count = 0;
		$query = 'SELECT survey_id FROM esy_surveys WHERE is_draft = 1 AND rec_status_id = 1;';
		$db_result = $DBH->query($query);
		if(!DB::isError($db_result)){ $survey_count = $db_result->numRows(); }
		return $survey_count;
	}
	// --------------------------------------------------------------------------
	// This function returns the number of survey's waiting for launch 
	function get_survey_waiting_count(){
		global $DBH;
		$survey_count = 0;
		$query = 'SELECT survey_id FROM esy_surveys WHERE is_draft = 0 AND start_timestamp > '.time().' AND rec_status_id = 1;';
		$db_result = $DBH->query($query);
		if(!DB::isError($db_result)){ $survey_count = $db_result->numRows(); }
		return $survey_count;
	}
	// --------------------------------------------------------------------------
	// This function returns the number of survey's currently running 
	function get_survey_running_count(){
		global $DBH;
		$survey_count = 0;
		$query = 'SELECT survey_id FROM esy_surveys WHERE is_draft = 0 AND start_timestamp < '.time().' AND end_timestamp > '.time().' AND rec_status_id = 1;';
		$db_result = $DBH->query($query);
		if(!DB::isError($db_result)){ $survey_count = $db_result->numRows(); }
		return $survey_count;
	}
	// --------------------------------------------------------------------------
	// This function returns the number of survey's that have been completed
	// (current time is past the end time)
	function get_survey_completed_count(){
		global $DBH;
		$survey_count = 0;
		$query = 'SELECT survey_id FROM esy_surveys WHERE is_draft = 0 AND end_timestamp < '.time().' AND rec_status_id = 1;';
		$db_result = $DBH->query($query);
		if(!DB::isError($db_result)){ $survey_count = $db_result->numRows(); }
		return $survey_count;
	}
	// --------------------------------------------------------------------------
	// This function returns an array of survey_ids with a draft status
	function get_draft_survey_ids(){
		global $DBH;
		$survey_ids = Array();
		$query = 'SELECT survey_id FROM esy_surveys WHERE is_draft = 1 AND rec_status_id = 1;';
		$db_result = $DBH->query($query);
		while($db_row = $db_result->fetchRow()){ $survey_ids[] = $db_row['survey_id']; }
		return $survey_ids;
	}
	// --------------------------------------------------------------------------
	// This function returns an array of survey_ids with a waiting status
	function get_waiting_survey_ids(){
		global $DBH;
		$survey_ids = Array();
		$query = 'SELECT survey_id FROM esy_surveys WHERE is_draft = 0 AND start_timestamp > '.time().' AND rec_status_id = 1;';
		$db_result = $DBH->query($query);
		while($db_row = $db_result->fetchRow()){ $survey_ids[] = $db_row['survey_id']; }
		return $survey_ids;
	}
	// --------------------------------------------------------------------------
	// This function returns an array of survey_ids with a running status
	function get_running_survey_ids(){
		global $DBH;
		$survey_ids = Array();
		$query = 'SELECT survey_id FROM esy_surveys WHERE is_draft = 0 AND start_timestamp < '.time().' AND end_timestamp > '.time().' AND rec_status_id = 1;';
		$db_result = $DBH->query($query);
		while($db_row = $db_result->fetchRow()){ $survey_ids[] = $db_row['survey_id']; }
		return $survey_ids;
	}
	// --------------------------------------------------------------------------
	// This function returns an array of survey_ids with a completed status
	function get_completed_survey_ids(){
		global $DBH;
		$survey_ids = Array();
		$query = 'SELECT survey_id FROM esy_surveys WHERE is_draft = 0 AND end_timestamp < '.time().' AND rec_status_id = 1;';
		$db_result = $DBH->query($query);
		while($db_row = $db_result->fetchRow()){ $survey_ids[] = $db_row['survey_id']; }
		return $survey_ids;
	}
	// --------------------------------------------------------------------------
	// this function takes a survey id and finds the number of unique responses
	// that have been logged for the survey
	function get_response_count($survey_id){
		global $DBH;
		$response_count = 0;
		if(strlen($survey_id) > 0 && is_numeric($survey_id)){
			$query = "SELECT COUNT(response_id) AS responses FROM esy_responses WHERE survey_id = $survey_id AND rec_status_id = 1;"; 
			$db_result = $DBH->query($query);
			if(!DB::isError($db_result)){ 
				$db_row = $db_result->fetchRow();
				$response_count = $db_row['responses'];
			}
		}
		return $response_count;
	}
	// --------------------------------------------------------------------------
	// this function finds the total number of responses for a given question.
	// because text and option answers are stored in different tables, we must
	// check the question_type_id to draw data from the correct table
	function get_question_response_count($survey_id, $question_id, $question_type_id){
		global $DBH;
		global $TID_TEXT_BOX, $TID_TEXT_AREA;
		$response_count = 0;
		if(strlen($survey_id) > 0 && is_numeric($survey_id) && strlen($question_id) > 0 && is_numeric($question_id)){
			// choose the appropriate DB table to draw the count from
			if($question_type_id == $TID_TEXT_BOX || $question_type_id == $TID_TEXT_AREA){ $source_table = 'esy_text_answers'; }
			else{ $source_table = 'esy_answers'; }

			$query = "SELECT COUNT(DISTINCT response_id) AS responses FROM $source_table WHERE survey_id = $survey_id AND question_id = $question_id AND rec_status_id = 1;"; 
			$db_result = $DBH->query($query);
			if(!DB::isError($db_result)){ 
				$db_row = $db_result->fetchRow();
				$response_count = $db_row['responses'];
			}
		}
		return $response_count;
	}
	// --------------------------------------------------------------------------
	// this function finds the number of unique responses to a single question's
	// option in a survey (for pulldowns, radio buttons, etc.)
	function get_option_response_count($survey_id, $question_id, $option_id){
		global $DBH;
		global $TID_TEXT_BOX, $TID_TEXT_AREA;
		$response_count = 0;
		if(strlen($survey_id) > 0 && is_numeric($survey_id) && strlen($question_id) > 0 && is_numeric($question_id) && strlen($option_id) > 0 && is_numeric($option_id)){
			$query = "SELECT COUNT(DISTINCT response_id) AS responses FROM esy_answers WHERE survey_id = $survey_id AND question_id = $question_id AND option_id = $option_id AND rec_status_id = 1;"; 
			$db_result = $DBH->query($query);
			if(!DB::isError($db_result)){ 
				$db_row = $db_result->fetchRow();
				$response_count = $db_row['responses'];
			}
		}
		return $response_count;
	}
	// --------------------------------------------------------------------------
	// This function marks a survey and any questions and options associated
	// with it as being deleted (rec_status_id set to 0). Assuming no db errors
	// are thrown, a zero is returned if no survey could be located or a 1 if it
	// was successfully marked as deleted
	function delete_survey($survey_id){
		global $DBH;
		$ret_val = 0;
		// check for well formed survey_id
		if(strlen($survey_id) > 0 && is_numeric($survey_id)){
			// see if a non-deleted survey with that survey_id can be found in the DB
			if(get_survey_hash($survey_id) !== FALSE){

				// mark the survey as being deleted
				$query = "UPDATE esy_surveys SET rec_status_id = 0 WHERE survey_id = $survey_id;";
				$db_result = $DBH->query($query);
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }

				// find any questions associated with this survey
				$query = "SELECT question_id FROM esy_questions WHERE survey_id = $survey_id AND rec_status_id = 1";
				$db_result = $DBH->query($query);
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
				$valid_question_ids = Array(); // for tracking question_id's found in DB
				while($db_row = $db_result->fetchRow()){
					extract($db_row);
					array_push($valid_question_ids, $question_id);
				}

				if(count($valid_question_ids) > 0){
					// mark all of these questions as deleted in DB
					$query = "UPDATE esy_questions SET rec_status_id = 0 WHERE question_id IN (".join(",",$valid_question_ids).") AND survey_id = $survey_id AND rec_status_id = 1 ";
					$db_result = $DBH->query($query);
					if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }

					// mark all of these questions' options as deleted in DB
					$query = "UPDATE esy_options SET rec_status_id = 0 WHERE question_id IN (".join(",",$valid_question_ids).") AND rec_status_id = 1 ";
					$db_result = $DBH->query($query);
					if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
				}

				$ret_val = 1;
			}
		}
		return $ret_val;
	}
	// --------------------------------------------------------------------------
	// This function takes a survey_id, and if it finds it in the database, it
	// creates a new survey with the host survey's information. The newly cloned
	// survey has all new id's, is forced to draft status, has a default
	// start/end date, and has not statistics/responses associated with it. The
	// name is also modified accordingly based on how many other clones it
	// determines to already exist
	function clone_survey($host_survey_id){
		global $DBH;
		$cloned_survey_id = FALSE;

		// check for well formed survey_id
		if(strlen($host_survey_id) > 0 && is_numeric($host_survey_id)){
			// see if a non-deleted survey with that survey_id can be found in the DB
			if(get_survey_hash($host_survey_id) !== FALSE){

				// retrieve the relevant survey details
				$query = "SELECT survey_name,instructions,confirmation FROM esy_surveys WHERE survey_id = $host_survey_id;";
				$db_result = $DBH->query($query);
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
				$db_row = $db_result->fetchRow();
				extract($db_row);

				// look at the current survey name and see if it begins with "Copy
				// of". If it does, leave it alone, otherwise, add it to distinguish it from the original survey
				if(strpos("Copy of",$survey_name) !== 0){ $survey_name = "Copy of ".$survey_name; }

				// prep the data pulled from the DB for insertion into the DB
				$survey_name = to_db_str($survey_name);
				$instructions = to_db_str($instructions);
				$confirmation = to_db_str($confirmation);

				// predefine some of the cloned survey's attributes
				$start_timestamp = mktime(0,0,0,date("n",time()),date("j",time()),date("Y",time()));
				$end_timestamp = mktime(12,0,0,date("n",time()),date("j",time()),date("Y",time()));
				$is_draft = 1;

				// create a 32 char MD5 hash of the variable parts of our survey details. 
				$survey_hash = md5($survey_name.$start_timestamp.$end_timestamp.$is_draft.$instructions.$confirmation);

				// insert the new survey details into the DB
				$query = "INSERT INTO esy_surveys 
						  (survey_hash,survey_name,start_timestamp,end_timestamp,is_draft,instructions,confirmation,rec_status_id)
						  VALUES
						  ('$survey_hash','$survey_name',$start_timestamp,$end_timestamp,$is_draft,'$instructions','$confirmation',1);
				";
				$db_result = $DBH->query($query);
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }

				// get the newly created survey_id
				$cloned_survey_id = get_last_insert_id('survey_id','esy_surveys','survey_hash',$survey_hash);

				// construct an array of all of the question id's associated with our host survey
				$query = "SELECT question_id FROM esy_questions WHERE survey_id = $host_survey_id AND rec_status_id = 1";
				$db_result = $DBH->query($query);
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
				$host_question_ids = Array(); // for tracking question_id's found in DB
				while($db_row = $db_result->fetchRow()){
					array_push($host_question_ids, $db_row['question_id']);
				}

				// loop through each question_id one at a time
				for($i = 0; $i < count($host_question_ids); $i++){

					// get the details of each host question
					$query = "SELECT question_type_id,question_text,order_number,is_required,is_inc_stats,error_message 
							  FROM esy_questions WHERE question_id = ".$host_question_ids[$i]." AND survey_id = $host_survey_id AND rec_status_id = 1;";
					$db_result = $DBH->query($query);
					if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
					$db_row = $db_result->fetchRow();
					extract($db_row);

					// prep the data pulled from the DB for insertion into the DB
					$question_text = to_db_str($question_text);
					$error_message = to_db_str($error_message);

					// create a 32 char MD5 hash of the variable parts of our question details. 
					$question_hash = md5($question_type_id.$question_text.$order_number.$is_required.$is_inc_stats.$error_message);

					$query = "INSERT INTO esy_questions
							  (survey_id,question_hash,question_type_id,question_text,order_number,is_required,is_inc_stats,error_message,rec_status_id)
							  VALUES
							  ($cloned_survey_id,'$question_hash',$question_type_id,'$question_text',$order_number,$is_required,$is_inc_stats,'$error_message',1)
					";
					$db_result = $DBH->query($query);
					if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }

					// record the id that was inserted in the DB
					$cloned_question_id = get_last_insert_id('question_id','esy_questions','question_hash',$question_hash);

					// construct an array of all of the option id's associated with our host question 
					$query = "SELECT option_id FROM esy_options WHERE question_id = ".$host_question_ids[$i]." AND rec_status_id = 1";
					$db_result = $DBH->query($query);
					if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
					$host_option_ids = Array(); // for tracking option_id's found in DB
					while($db_row = $db_result->fetchRow()){
						array_push($host_option_ids, $db_row['option_id']);
					}

					// loop through each question_id one at a time
					for($j = 0; $j < count($host_option_ids); $j++){

						// get the details of each host question
						$query = "SELECT option_text,order_number 
								  FROM esy_options WHERE option_id = ".$host_option_ids[$j]." AND question_id = ".$host_question_ids[$i]." AND rec_status_id = 1;";
						$db_result = $DBH->query($query);
						if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
						$db_row = $db_result->fetchRow();
						extract($db_row);

						// prep the data pulled from the DB for insertion into the DB
						$option_text = to_db_str($option_text);

						// create a 32 char MD5 hash of the variable parts of our option details. 
						$option_hash = md5($option_text.$order_number);

						$query = "INSERT INTO esy_options
								  (question_id,option_hash,option_text,order_number,rec_status_id)
								  VALUES
								  ($cloned_question_id,'$option_hash','$option_text',$order_number,1)
						";
						$db_result = $DBH->query($query);
						if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }

						// record the option id that was inserted in the DB
						$cloned_option_id = get_last_insert_id('option_id','esy_options','option_hash',$option_hash);

					} // end for
				} // end for
			} // end if
		} // end if

		return $cloned_survey_id;
	}
	// ---------------------------------------------------------------------------
	// this function takes a survey_id and attempts to return an associative
	// array of questions, thier details, and any options they might have
	function get_question_data_struct($survey_id){
		global $DBH;

		// declare our variables
		$query = $db_result = $db_row = '';
		$questions_array = Array();

		// verify a well formed survey_id
		if(strlen($survey_id) > 0 && is_numeric($survey_id)){

			// create an array to store all of the question details
			$query = "SELECT question_id, question_type_id, question_text, is_required, is_inc_stats, error_message FROM esy_questions WHERE rec_status_id = 1 AND survey_id = $survey_id ORDER BY order_number";
			$db_result = $DBH->query($query);
			if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
			while($db_row = $db_result->fetchRow()){
				$db_row['options'] = Array(); // allocate a place to store options
				array_push($questions_array, $db_row);
			}

			// grab the option list for each question that is not a text field
			// (1) or a textarea (2)
			for($i = 0; $i < count($questions_array); $i++){
				if($questions_array[$i]['question_type_id'] > 2){
					$query = "SELECT option_id, option_text FROM esy_options WHERE question_id = ".$questions_array[$i]['question_id']." AND rec_status_id = 1 ORDER BY order_number";
					$db_result = $DBH->query($query);
					if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
					while($db_row = $db_result->fetchRow()){
						array_push($questions_array[$i]['options'], $db_row);
					}
				}
			} // end for
		} // end if

		return $questions_array;
	}
	// --------------------------------------------------------------------------
	// this function takes a survey_id and attempts to load all of the details
	// about the survey into an associative array. If nothing is found, then
	// returns an empty array
	function get_survey_data_struct($survey_id){
		global $DBH;

		// declare our variables
		$query = $db_result = '';
		$survey_struct = Array();

		// verify a well formed survey_id
		if(strlen($survey_id) > 0 && is_numeric($survey_id)){
			$query = "SELECT * FROM esy_surveys WHERE survey_id = $survey_id AND rec_status_id = 1";
			$db_result = $DBH->query($query);
			if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
			if($db_result->numRows() == 1){
				$survey_struct = $db_result->fetchRow();
			}
		} // end if

		return $survey_struct;
	}
	// --------------------------------------------------------------------------
	// this function grabs a list of valid file extensions from the DB and
	// returns them as strings in an array. each ext is lowercase and has no
	// leading period
	function get_valid_file_extensions(){
		global $DBH;
		// retrieve the list of file extensions from the DB
		$query = "SELECT pub_point_file_ext FROM esy_settings WHERE rec_status_id = 1 ORDER BY setting_id DESC";
		$db_result = $DBH->query($query);
		if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
		if($db_result->numRows() > 0){
			$db_row = $db_result->fetchRow();
			$valid_file_ext = $db_row['pub_point_file_ext'];
		}

		$ext_array = explode(';', $valid_file_ext);
		$return_array = Array();
		foreach($ext_array as $idx => $file_ext){
			// trim and lowercase the extensions
			$file_ext = trim(strtolower($file_ext));
			// strip the leading period if it exists
			if(strpos($file_ext,".") === 0){ $file_ext = substr($file_ext, 1); }
			if(strlen($file_ext) > 0){ $return_array[] = $file_ext; }
		} // end foreach

		return $return_array;
	}
	// --------------------------------------------------------------------------
	// this function takes a publish point id and attempts to find all of the
	// data associated with the publish point in the database. Upon error
	// boolean FALSE is returned, otherwise an associative array is returned
	// with the pub point details
	function get_pub_point_struct($publish_point_id){
		global $DBH;
		$pub_point_struct = FALSE;
		if(is_numeric($publish_point_id) && $publish_point_id > 0){ 
			// retrieve the pub point data from the DB
			$query = "SELECT * FROM esy_publish_points WHERE publish_point_id = ${publish_point_id} AND rec_status_id = 1";
			$db_result = $DBH->query($query);
			if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }

			// if the pub point was found
			if($db_result->numRows() == 1){
				$pub_point_struct = $db_result->fetchRow();
			}
		} // end if
		return $pub_point_struct;
	} // end function
	// --------------------------------------------------------------------------
	// this function takes a publish_point_id, a remote_filename string and the contents of a file and
	// stores it in the database's esy_file_archive table. This data never gets
	// used, but this is for disaster recovery scenarios. If a publish point
	// transfer goes bad we'll have the original file contents and where they came from
	function store_contents_to_file_archive($publish_point_id, $remote_filename, $file_contents){
		global $DBH;

		// prep our dat for insertion into the DB
		$publish_point_id = (is_numeric($publish_point_id)) ? $publish_point_id : 0;
		$timestamp = time();
		$remote_filename = to_db_str($remote_filename);
		$file_contents = to_db_str($file_contents);

		$query = "INSERT INTO esy_file_archive (publish_point_id, remote_filename, file_contents, timestamp) VALUES (${publish_point_id}, '${remote_filename}', '${file_contents}', ${timestamp})";
		$db_result = $DBH->query($query);
		if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }

		return TRUE;
	}
	// --------------------------------------------------------------------------
	// this function retrieves all of the application settings from database and
	// returns them in an associative array
	function get_settings_struct(){
		global $DBH;

		// declare our variables
		$query = $db_result = '';
		$settings_struct = Array();

		$query = "SELECT * FROM esy_settings WHERE rec_status_id = 1 ORDER BY setting_id DESC";
		$db_result = $DBH->query($query);
		if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
		if($db_result->numRows() > 0){
			$settings_struct = $db_result->fetchRow();
		}

		return $settings_struct;
	}
	// --------------------------------------------------------------------------
?>
