<?php
	// ---------------------------------------------------------------------------
	// this function takes all of the output from the script, formats it as a
	// javascript string and then outputs a document.write() call to display the
	// content generated by the script
	function callback_buffer_to_js_str($buffer){
		$buffer = to_js_str($buffer);
		$js_code = "
			document.write(\"$buffer\");
		";
		return $js_code;
	}
	// ---------------------------------------------------------------------------
	// this function takes an array of question structs and generates the
	// javascript code necessary to validate the form.
	function get_validation_js_code($questions_array){
		$js_code = "
			function esy_validate_survey(){
				var is_valid = 1;
		";
		foreach($questions_array as $question_num => $question_struct){
			// only create checking code for required questions
			if($question_struct['is_required'] == 1){

				// choose which js validation function will be called
				$js_function = '';
				switch($question_struct['question_type_id']){
					case 1: $js_function = "esy_validate_text_field($question_num)"; break;
					case 2: $js_function = "esy_validate_textarea($question_num)"; break;
					case 3: $js_function = "esy_validate_radio_buttons($question_num)"; break;
					case 4: $js_function = "esy_validate_check_boxes(".count($question_struct['options']).",$question_num)"; break;
					case 5: $js_function = "esy_validate_select($question_num)"; break;
					case 6: $js_function = "esy_validate_select($question_num)"; break;
				}

				// output the standard field checking code
				$js_code .= "
					if($js_function == 0){
						esy_set_error_text($question_num,\"".to_js_str($question_struct['error_message'])."\");
						is_valid = 0;
					}
				";
			} // end if
		} // end foreach
		$js_code .= "
				return is_valid;
			}
		";
		return $js_code;
	}
	// ---------------------------------------------------------------------------
	function get_text_field_html($question_num, $question_struct){
		$out_html = "";
		$out_html .= "<input class='esy_text_field' type='text' name='esy_q${question_num}_ans' id='esy_q${question_num}_ans' value='' maxlength=''><br>\n";
		return $out_html;
	}
	// ---------------------------------------------------------------------------
	function get_textarea_html($question_num, $question_struct){
		$out_html = "";
		$out_html .= "<textarea name='esy_q${question_num}_ans' id='esy_q${question_num}_ans' class='esy_textarea'></textarea><br>\n";
		return $out_html;
	}
	// ---------------------------------------------------------------------------
	// this function takes either a radio button question and
	// returns the html needed to render them in a nice grid/table format
	function get_radio_button_html($question_num, $question_struct){
		$out_html = "";
		$output_grid = Array(); // 2D array to store table cell contents
		$options = $question_struct['options'];
		$num_cols = get_num_option_columns($options);
		$num_options = count($options);
		$items_per_col = floor($num_options / $num_cols);

		// give the output grid the right number of dimensions wide
		for($i = 0; $i < $num_cols; $i++){ $output_grid[$i] = Array(); }

		// place the radio button html into the 2D array
		$offset_idx = 0;
		for($x = 0; $x < $num_cols; $x++){
			for($y = 0; $y < $items_per_col; $y++){
				if($offset_idx < $num_options){
					$output_grid[$x][$y] = "
						<td align='center' valign='middle'><input class='esy_form_radio' type='radio' name='esy_q${question_num}_ans' value='".$options[$offset_idx]['option_id']."'><br></td>
						<td align='left' valign='middle'><div class='esy_option_text'>".$options[$offset_idx]['option_text']."<br></div></td>
					";
				}
				else{ $output_grid[$x][$y] = "<td><br></td>\n<td><br></td>\n"; }
				$offset_idx++;
			}
		}

		// loop through the 2D array to generate the table html
		$out_html .= "<table cellspacing='0' cellpadding='0' border='0'>\n";
		for($y = 0; $y < $items_per_col; $y++){
			$out_html .= "<tr>\n";
			for($x = 0; $x < $num_cols; $x++){ $out_html .= $output_grid[$x][$y]; }
			$out_html .= "</tr>\n";
		}
		$out_html .= "</table>\n";

		return $out_html;
	}
	// ---------------------------------------------------------------------------
	// this function takes either a check box question and
	// returns the html needed to render them in a nice grid/table format
	function get_check_box_html($question_num, $question_struct){
		$out_html = "";
		$output_grid = Array(); // 2D array to store table cell contents
		$options = $question_struct['options'];
		$num_cols = get_num_option_columns($options);
		$num_options = count($options);
		$items_per_col = floor($num_options / $num_cols);

		// give the output grid the right number of dimensions wide
		for($i = 0; $i < $num_cols; $i++){ $output_grid[$i] = Array(); }

		// place the check boxhtml into the 2D array
		$offset_idx = 0;
		for($x = 0; $x < $num_cols; $x++){
			for($y = 0; $y < $items_per_col; $y++){
				if($offset_idx < $num_options){
					$output_grid[$x][$y] = "
						<td align='center' valign='middle'><input class='esy_form_checkbox' type='checkbox' name='esy_q${question_num}_ans[]' value='".$options[$offset_idx]['option_id']."'><br></td>
						<td align='left' valign='middle'><div class='esy_option_text'>".$options[$offset_idx]['option_text']."<br></div></td>
					";
				}
				else{ $output_grid[$x][$y] = "<td><br></td>\n<td><br></td>\n"; }
				$offset_idx++;
			}
		}

		// loop through the 2D array to generate the table html
		$out_html .= "<table cellspacing='0' cellpadding='0' border='0'>\n";
		for($y = 0; $y < $items_per_col; $y++){
			$out_html .= "<tr>\n";
			for($x = 0; $x < $num_cols; $x++){ $out_html .= $output_grid[$x][$y]; }
			$out_html .= "</tr>\n";
		}
		$out_html .= "</table>\n";

		return $out_html;
	}
	// ---------------------------------------------------------------------------
	// this function retrieves the html necessary to display a dropdown widget
	// in the survey being rendered.
	function get_drop_down_html($question_num, $question_struct){
		$out_html = "";
		$options = $question_struct['options'];
		$num_options = count($options);

		$out_html .= "<select name='esy_q${question_num}_ans' id='esy_q${question_num}_ans' size='1' class='esy_form_select'>
						<option value=''>Choose One...</option>\n";
		foreach($options as $idx => $opt){
			$out_html .= "<option value='".$opt['option_id']."'>".$opt['option_text']."</option>\n";
		}
		$out_html .= "</select>\n";

		return $out_html;
	}
	// ---------------------------------------------------------------------------
	// this function retrieves the html necessary to display a select box widget
	// in the survey being rendered.
	function get_select_box_html($question_num, $question_struct){
		$out_html = "";
		$options = $question_struct['options'];
		$num_options = count($options);

		$out_html .= "<select name='esy_q${question_num}_ans[]' id='esy_q${question_num}_ans' size='8' multiple class='esy_form_select'>\n";
		foreach($options as $idx => $opt){
			$out_html .= "<option value='".$opt['option_id']."'>".$opt['option_text']."</option>\n";
		}
		$out_html .= "</select>\n";

		return $out_html;
	}
	// ---------------------------------------------------------------------------
	// this function takes an array of options structs and tries to determine
	// the best number of columns to use display them based on the number of
	// options and the string length of the longest option
	function get_num_option_columns($option_array){
		$num_cols = 1;
		$max_str_len = 0;

		if(is_array($option_array)){
			$num_options = count($option_array);
				
			// if there are more than 5 options, then the question may require
			// more than one column of option fields. 
			if($num_options > 5){
				// find the length of the longest option string
				for($i = 0; $i < count($option_array); $i++){
					if(strlen($option_array[$i]['option_text']) > $max_str_len){ $max_str_len = strlen($option_array[$i]['option_text']); }
				}
				// long strings cause fewer columns to be displayed
				if($max_str_len > 40){ $num_cols = 1; }
				elseif($max_str_len > 20){ $num_cols = 2; }
				else{ $num_cols = 3; }
			}
		}
		return $num_cols;
	}
	// ---------------------------------------------------------------------------
	// this function checks the database to see if the survey matching the
	// survey_id passed in exists, is active, and is currently running. Returns
	// 1 on success, 0 on failure
	function verify_running_survey($survey_id){
		global $DBH;

		// declare our variables
		$query = $db_result = $db_row = '';
		$ret_val = 0;

		// verify a well formed survey_id
		if(strlen($survey_id) > 0 && is_numeric($survey_id)){
			// verify the survey_id appears in the database, is running, and is active
			$query = "SELECT survey_id FROM esy_surveys WHERE is_draft = 0 AND rec_status_id = 1 AND start_timestamp < ".time()." AND end_timestamp > ".time()." AND survey_id = $survey_id";
			$db_result = $DBH->query($query);
			if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
			if($db_result->numRows() == 1){ $ret_val = 1; }
		} 
		return $ret_val;
	}
	// ---------------------------------------------------------------------------
	// this function simply returns all of the common javascript code necessary to do
	// survey form validation
	function get_global_survey_js_code(){
		$js_code = "
			var esy_submit_in_progress = 0;
			// ---------------------------------------------------------------------------
			function esy_submit_form(_question_count){
				var info_msg = '';
				var f = document.forms['esy_form'];

				if(!esy_submit_in_progress){
					esy_submit_in_progress = 1;

					esy_clear_all_errors(_question_count);
					if(esy_validate_survey() == 0){
						esy_submit_in_progress = 0;
					}
					else{ f.submit(); }
				}
			}
			// ---------------------------------------------------------------------------
			// this function fakes a form submit for previewing surveys. Error
			// checking is done. Upon success, we skip the stat collection and
			// simply display the thankyou message.
			function esy_preview_form(_question_count){
				var f = document.forms['esy_form'];

				esy_clear_all_errors(_question_count);
				if(esy_validate_survey() != 0){
					document.location = document.URL+'&esy_thank_you=1';
				}
			}
			// ---------------------------------------------------------------------------
			// when the survey is rendered on the client webpage, this function grabs
			// the current url and passes it along to our stat collecting script so it
			// knows where to return to.
			function esy_set_host_url(){
				if(document.getElementById){
					var hidden_input = document.getElementById('esy_host_url');
					if(hidden_input){ hidden_input.value = document.URL; }
				}
			}
			// ---------------------------------------------------------------------------
			// this function attempts to validate that a text field has been filled out
			function esy_validate_text_field(_question_num){
				var ret_val = 0;
				var dom_obj = null;
				var txt = '';
				if(document.getElementById){
					dom_obj = document.getElementById('esy_q'+_question_num+'_ans');
					if(dom_obj){
						txt = esy_trim(dom_obj.value);
						if(txt.length > 0){ ret_val = 1; }
					}
				}
				return ret_val;
			}
			// ---------------------------------------------------------------------------
			// this function attempts to validate that a text field has been filled out
			function esy_validate_textarea(_question_num){
				var f = document.forms['esy_form'];
				var ret_val = 0;
				var dom_obj = null;
				var txt = '';
				if(f){
					dom_obj = f['esy_q'+_question_num+'_ans'];
					if(dom_obj){
						txt = esy_trim(dom_obj.value);
						if(txt.length > 0){ ret_val = 1; }
					}
				}
				return ret_val;
			}
			// ---------------------------------------------------------------------------
			// this function attempts to validate that one of the radio button options
			// for a question has been selected
			function esy_validate_radio_buttons(_question_num){
				var ret_val = 0;
				var dom_obj = null;
				var f = document.forms['esy_form'];
				var i = 0;
				if(f){
					dom_obj = f['esy_q'+_question_num+'_ans'];
					if(dom_obj){
						for(i = 0; i < dom_obj.length; i++){
							if(dom_obj[i].checked){ ret_val = 1; break; }
						}
					}
				}
				return ret_val;
			}
			// ---------------------------------------------------------------------------
			// this function attempts to validate that at least one of the check box options
			// for a question has been selected
			function esy_validate_check_boxes(_option_count,_question_num){
				var ret_val = 0;
				var dom_obj = null;
				var f = document.forms['esy_form'];
				var i = 0;
				if(f){
					dom_obj = f['esy_q'+_question_num+'_ans[]'];
					if(dom_obj){
						for(i = 0; i < dom_obj.length; i++){
							if(dom_obj[i].checked){ ret_val = 1; break; }
						}
					}
				}
				return ret_val;
			}
			// ---------------------------------------------------------------------------
			// this function attempts to validate that at least one of the options
			// for a question has been selected for either a select box or drop down widget
			function esy_validate_select(_question_num){
				var ret_val = 0;
				var dom_obj = null;
				var f = document.forms['esy_form'];
				var i = 0;
				if(f){
					dom_obj = f['esy_q'+_question_num+'_ans'];
					if(dom_obj){
						for(i = 0; i < dom_obj.length; i++){
							if(dom_obj[i].selected && dom_obj[i].value != ''){ ret_val = 1; break; }
						}
					}
				}
				return ret_val;
			}
			// ---------------------------------------------------------------------------
			// this function takes a string and trims off any white space at the
			// beginning and end of the string
			function esy_trim(_str){
				if(_str != null && _str != void(0) && _str != ''){ _str = _str.replace(/^\s*/,'').replace(/\s*$/,''); }
				else{ _str = ''; }
				return _str;
			}
			// --------------------------------------------------------------------------
			// this function displays an error message beneath the question text on a
			// survey.
			function esy_set_error_text(_question_num, _error_msg){
				var dom_obj = null;
				if(document.getElementById){
					dom_obj = document.getElementById('esy_q'+_question_num+'_error');
					if(dom_obj){ dom_obj.innerHTML = _error_msg; }
				}
			}
			// --------------------------------------------------------------------------
			// this function loops through all of the questions and makes sure that the
			// error text is turned off/hidden
			function esy_clear_all_errors(_question_count){
				var dom_obj = null;
				var i = 0;
				if(document.getElementById){
					for(i = 0; i < _question_count; i++){
						dom_obj = document.getElementById('esy_q'+i+'_error');
						if(dom_obj){ dom_obj.innerHTML = ''; }
					}
				}
			}
			// --------------------------------------------------------------------------
		";
		return $js_code;
	} // end function
	// ---------------------------------------------------------------------------
	// this function attempts to get the submit count data structure stored in
	// the clients cookie. On success, the datastruct array is returned. If no
	// cookie is found, an empty array is returned.
	function get_submit_count_struct(){
		// this variable is an associative array where keys are survey ids and
		// values are the number of times a survey has been submitted. This
		// variable is stored and maintined in a cookie
		$submit_count_struct = Array();

		// first, see if a cookie has been set up for tracking survey submissions
		if(isset($_COOKIE['esy_submit_cnt'])){
			// make sure our cookie is in exactly the right format. This is absolutely
			// necessary, otherwise, a user could tamper with thier cookie and allow for
			// the execution of arbitrary PHP code (really bad).
			if(preg_match('/^(\$s\[\d+\]=\d+;)*$/',$_COOKIE['esy_submit_cnt'])){
				// the cookie stores all values under a variable called $s. A short
				// nondescript variable name is used due to the space limitations
				// imposed by cookies
				$s = Array();
				// by evaling the cookie, we can quickly place all of the
				// survey_id's and sumit counts into the array
				eval($_COOKIE['esy_submit_cnt']);
				// now we use a more friendly variable name
				$submit_count_struct = $s;
			} // end if
		} // end if
		return $submit_count_struct;
	}
	// ---------------------------------------------------------------------------
	// this function takes a submit count data struct and stores that data in
	// the form of a cookie on the client machine. The data is trimmed so that
	// no extra space is used in the cookie for surveys that have already
	// completed.
	function set_submit_count_cookie($submit_count_struct){
		global $DBH;
		if(is_array($submit_count_struct)){
			// We need to trim the cookie and make sure that it isn't
			// storing any count data about surveys that have ended. This is due to the
			// size limit imposed upon cookies, so we need to keep them lean.
			if(count($submit_count_struct) > 0){
				$new_submit_count = Array();
				$query = "SELECT survey_id FROM esy_surveys 
						  WHERE survey_id IN (".implode(",", array_keys($submit_count_struct)).") 
						  AND end_timestamp > ".time()." AND rec_status_id = 1";
				$db_result = $DBH->query($query);
				if($db_result->numRows() > 0){
					while($db_row = $db_result->fetchRow()){ 
						$new_submit_count[$db_row['survey_id']] = $submit_count_struct[$db_row['survey_id']];
					}
				} // end if
				$submit_count_struct = $new_submit_count;
			} // end if
			// the last step is to store the updated count data back to our cookie
			// so we have to loop through and construct a serialized version of our
			// count data struct
			$cookie_value = "";
			foreach($submit_count_struct as $key => $val){
				$cookie_value .= '$s['.$key.']='.$val.';';
			}
			// make the cookie valid for a year 
			setcookie('esy_submit_cnt', $cookie_value, time()+31536000);
		} // end if
	}
	// ---------------------------------------------------------------------------
?>
