<?php
	// --------------------------------------------------------------------------
	// this function generates the html for the body of the survey distribution
	// report which shows how many people answered/skipped questions and the
	// answers to those questions.
	function get_survey_distrib_report_html($survey_id, $response_count){
		global $TID_TEXT_BOX, $TID_TEXT_AREA, $TID_RADIO_BUTTONS, $TID_CHECK_BOXES, $TID_DROP_DOWN, $TID_SELECT_BOX;

		// colors for the percent answered bars
		$skipped_rgb = '#0058a3';
		$answered_rgb = '#0058a3';
		$option_rgb_array = Array('#ffd200');
		//$option_rgb_array = Array('#ffd200','#17a300','#0058a3','#9b49b2');

		$out_html = '';

		// fetch all of the question data associated with this survey
		$questions_array = get_question_data_struct($survey_id);

		// go through each question looking for response/answer data
		foreach($questions_array as $idx => $question_struct){
			$question_num = $idx + 1;
			$is_required = ($question_struct['is_required'] == 1) ? '*' : '';
			$rgb_idx = 0;

			// find out how many people responded to just this question
			$question_response_count = get_question_response_count($survey_id, $question_struct['question_id'], $question_struct['question_type_id']);

			if($response_count == 0){ 
				$percent_skipped = 0; 
				$percent_responded = 0;
			}
			else{ 
				$percent_skipped = round(($response_count - $question_response_count) / $response_count * 100);
				$percent_responded = round($question_response_count / $response_count * 100);
			}

			// see if the question is marked for display in the report
			if($question_struct['is_inc_stats'] == 1){

				// get the base img name and alt text for this type of question
				$qtype_name = '';
				$qtype_alt = '';
				switch($question_struct['question_type_id']){
					case $TID_TEXT_BOX: $qtype_name = 'text_box'; $qtype_alt = 'Text Box'; break;
					case $TID_TEXT_AREA: $qtype_name = 'text_area'; $qtype_alt = 'Text Area'; break;
					case $TID_RADIO_BUTTONS: $qtype_name = 'radio_buttons'; $qtype_alt = 'Radio Buttons'; break;
					case $TID_CHECK_BOXES: $qtype_name = 'check_boxes'; $qtype_alt = 'Check Boxes'; break;
					case $TID_DROP_DOWN: $qtype_name = 'drop_down'; $qtype_alt = 'Drop Down'; break;
					case $TID_SELECT_BOX: $qtype_name = 'select_box'; $qtype_alt = 'Select Box'; break;
				} // end switch
				$q_icon_img = "<img align='left' width='28' height='21' src='./images/reports/icon/icon_${qtype_name}_off.gif' alt='${qtype_alt}' title='${qtype_alt}' hspace='0' vspace='0' border='0' style='margin-right: 9px; background-color: #ededef; border-right: 1px solid #8f99a3;'>";

				$out_html .= "
					<tr>
						<td class='l_question_cell l_table_b_border' colspan='3' height='22' align='left' valign='middle'>
							<div class='l_table_text' style='width:640px;' title='".to_html_attrib_str($question_struct['question_text'])."'>
							$q_icon_img<div style='margin-top:4px;'><b>(${question_num})</b> <span class='l_required'>$is_required</span><b>".$question_struct['question_text']."</b></div>
							</div>
						</td>
					</tr>
					<tr>
						<td class='l_sub_header_cell l_table_r_border l_table_b_border' width='248' height='22' align='left' valign='middle'><div class='l_table_text' style='width:230px; margin-left: 16px;'><b>People Who Skipped Question:</b></div></td>
						<td class='l_sub_header_cell l_table_r_border l_table_b_border' width='350' height='22'>".get_percent_bar_html($percent_skipped, $skipped_rgb)."</td>
						<td class='l_sub_header_cell l_table_b_border' width='94' height='22' align='center' valign='middle'><div class='l_table_text'>".($response_count - $question_response_count)."<br></div></td>
					</tr>
					<tr>
						<td class='l_sub_header_cell l_table_r_border l_table_b_border' width='248' height='22' align='left' valign='middle'><div class='l_table_text' style='width:230px; margin-left: 16px;'><b>People Who Answered Question:</b></div></td>
						<td class='l_sub_header_cell l_table_r_border l_table_b_border' width='350' height='22'>".get_percent_bar_html($percent_responded, $answered_rgb)."</td>
						<td class='l_sub_header_cell l_table_b_border' width='94' height='22' align='center' valign='middle'><div class='l_table_text'>$question_response_count<br></div></td>
					</tr>
				";

				switch($question_struct['question_type_id']){
					case $TID_TEXT_BOX: case $TID_TEXT_AREA:
					break;
					case $TID_RADIO_BUTTONS: case $TID_CHECK_BOXES: case $TID_DROP_DOWN: case $TID_SELECT_BOX:
						foreach($question_struct['options'] as $option_idx => $option_struct){
							$option_response_count = get_option_response_count($survey_id, $question_struct['question_id'], $option_struct['option_id']);
							if($question_response_count == 0){ $percent_answered = 0; }
							else{ $percent_answered = round($option_response_count/$question_response_count*100); }
							$out_html .= "
								<tr>
									<td class='l_stat_cell l_table_r_border l_table_b_border' width='248' height='22' align='left' valign='middle'><div class='l_table_text' style='width:223px; margin-left: 16px;' title='".to_html_attrib_str($option_struct['option_text'])."'>".$option_struct['option_text']."</div></td>
									<td class='l_stat_cell l_table_r_border l_table_b_border' width='350' height='22'>".get_percent_bar_html($percent_answered, $option_rgb_array[$rgb_idx])."</td>
									<td class='l_stat_cell l_table_b_border' width='94' height='22' align='center' valign='middle'><div class='l_table_text'>$option_response_count<br></div></td>
								</tr>
							";

							// increment the color of the percent bars
							$rgb_idx++;
							if($rgb_idx == count($option_rgb_array)){ $rgb_idx = 0; }

						} // end foreach
					break;
				} // end switch
			} // end if
		} // end foreach
		return $out_html;
	}
	// --------------------------------------------------------------------------
	// this function takes a number from 0 to 100 and returns the html necessary
	// to display the report's percent complete bar
	function get_percent_bar_html($percent, $rgb_color){
		$out_html = '';
		if(strlen($percent) > 0 && is_numeric($percent) && $percent >= 0 && $percent <= 100){
			$out_html = "
				<table cellspacing='0' cellpadding='0' border='0'>
				<tr>
				<td width='282' align='left' valign='middle'><div class='l_percent_bar'><img style='background-color: ${rgb_color};'width='${percent}%' height='13' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'></div></td>
					<td width='54' align='center' valign='middle'><div class='l_table_text'>${percent}%<br></div></td>
				</tr>
				</table>
			";
		} // end if
		return $out_html;
	}
	// --------------------------------------------------------------------------
	// this function generates the javascript data structure needed to
	// dynamically update the survey name pulldown menu on the reports page
	function get_survey_pulldown_js_struct(){
		$out_html= '';
		$out_html .= "
			// store each survey name and id in an object by their
			// status group (draft, waiting, etc.)
			var spd = new Array(); // temp variable for (survey pull down)
		";

		for($i = 0; $i < 4; $i++){
			switch($i){
				case 0: $survey_ids = get_draft_survey_ids(); break;
				case 1: $survey_ids = get_waiting_survey_ids(); break;
				case 2: $survey_ids = get_running_survey_ids(); break;
				case 3: $survey_ids = get_completed_survey_ids(); break;
			} // end switch

			$out_html .= "spd[${i}] = new Array();\n";
			foreach($survey_ids as $idx => $pulldown_survey_id){
				$survey_struct = get_survey_data_struct($pulldown_survey_id);
				$out_html .= "spd[${i}][${idx}] = new Object();\n";
				$out_html .= "spd[${i}][${idx}]['survey_id'] = ".$survey_struct['survey_id'].";\n";
				$out_html .= "spd[${i}][${idx}]['survey_name'] = '".to_js_str($survey_struct['survey_name'])."';\n";
			}
		} // end for

		$out_html .= "
			var survey_pulldown = spd;
		";
		return $out_html;
	}
	// --------------------------------------------------------------------------
?>
