<?php
	ob_start();

	require_once('../pear/DB.php');
	include_once('../settings/settings.php');
	include_once('../include/globals.php');
	include_once('../include/loader.php');
	include_once('../include/login_functions.php');
	// make sure the user has logged in before accessing this page
	if(($user_struct = get_login_status()) === FALSE){ header("Location: ../login.php"); exit(0); }

	include_once('../include/functions.php');

	// place to store all error messages
	$error_log = Array();

	$csv_data_array = Array();
	$csv_data_str = "";

	// make sure all variables that may be passed in get declared
	$survey_id = isset($_REQUEST['survey_id']) ? $_REQUEST['survey_id'] : '';

	// see if a valid survey id was passed in. Redirect back to survey list page if not
	if(get_survey_hash($survey_id) === FALSE){ header("Location: survey_list.php"); exit(0); }

	// fetch all of our question data
	$questions_array = get_question_data_struct($survey_id);

	// loop through each question and add a column to the csv array for each.
	foreach($questions_array as $question_num => $question_struct){
		// each column in the data structure is an array of values
		$csv_data_array[$question_num] = Array();

		// the first value in each column is the actual survey question
		$csv_data_array[$question_num][] = $question_struct['question_text'];
	}

	// loop through each question and depending upon it's type, construct the
	// data for csv data column
	foreach($questions_array as $question_num => $question_struct){
		switch($question_struct['question_type_id']){
			case $TID_RADIO_BUTTONS: case $TID_DROP_DOWN:
				$query = "SELECT r.response_id, r.response_timestamp, o.option_text
						  FROM esy_responses r 
						  LEFT JOIN esy_answers a ON r.response_id = a.response_id AND r.survey_id = ${survey_id} AND a.question_id = ".$question_struct['question_id']." 
						  LEFT JOIN esy_options o ON a.option_id = o.option_id
					  	  WHERE r.rec_status_id = 1 AND (a.rec_status_id = 1 OR a.rec_status_id IS NULL)
						  ORDER BY r.response_timestamp ASC";
				$db_result = $DBH->query($query);
				// place the response data into the correct csv data column
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
				while($db_row = $db_result->fetchRow()){
					// trim and fix quotes to two quotes in csv data
					$csv_data_array[$question_num][] = trim(str_replace('"','""',$db_row['option_text']));
				}
				$db_result->free();
			break;
			case $TID_TEXT_BOX: case $TID_TEXT_AREA:
				$query = "SELECT r.response_id, r.response_timestamp, ta.answer_text
						  FROM esy_responses r 
						  LEFT JOIN esy_text_answers ta ON r.response_id = ta.response_id AND r.survey_id = ${survey_id} AND ta.question_id = ".$question_struct['question_id']." 
						  WHERE r.rec_status_id = 1 AND (ta.rec_status_id = 1 OR ta.rec_status_id IS NULL)
						  ORDER BY r.response_timestamp ASC";
				$db_result = $DBH->query($query);
				// place the response data into the correct csv data column
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
				while($db_row = $db_result->fetchRow()){
					// trim and strip out newline characters from the string
					$csv_data_array[$question_num][] = trim(str_replace("\n"," ",str_replace("\r"," ",str_replace('"','""',$db_row['answer_text']))));
				}
				$db_result->free();
			break;
			case $TID_CHECK_BOXES: case $TID_SELECT_BOX:
				$response_map_array = Array();
				$query = "SELECT r.response_id, r.response_timestamp, o.option_text
						  FROM esy_responses r 
						  LEFT JOIN esy_answers a ON r.response_id = a.response_id AND r.survey_id = ${survey_id} AND a.question_id = ".$question_struct['question_id']." 
						  LEFT JOIN esy_options o ON a.option_id = o.option_id
					  	  WHERE r.rec_status_id = 1 AND (a.rec_status_id = 1 OR a.rec_status_id IS NULL)
						  ORDER BY r.response_timestamp ASC";
				$db_result = $DBH->query($query);
				// place the response data into the correct csv data column
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
				while($db_row = $db_result->fetchRow()){
					$key_name = $db_row['response_timestamp'];
					if(!isset($response_map_array[$key_name])){ $response_map_array[$key_name] = Array(); }
					// make sure the string is trimmed and doesn't contain any pipe characters
					$response_map_array[$key_name][] = trim(str_replace("|","",str_replace('"','""',$db_row['option_text'])));
				}
				$db_result->free();
				// pipe seperate the responses to generate a single line for
				// each response in the csv data struct
				foreach($response_map_array as $i => $response_array){
					$csv_data_array[$question_num][] = implode("|", $response_array);
				}
			break;
		} // end switch
	} // end foreach

	// now that the csv data struct has been made, we need to flatten it into a
	// single csv data string to return to the user
	for($row = 0; $row < count($csv_data_array[0]); $row++){
		for($col = 0; $col < count($csv_data_array); $col++){

			// add the column seperator comma
			if($col != 0){ $csv_data_str .= ","; }
			// non-numeric values get enclosed by quotes
			if(!is_numeric($csv_data_array)){ $csv_data_str .= '"'.$csv_data_array[$col][$row].'"'; }
			// numeric data stays as is
			else{ $csv_data_str .= $csv_data_array[$col][$row]; }

		} // end for
		// carriage return signals a row seperator
		$csv_data_str .= "\n";
	} // end for

	// send the download headers 
	header("Cache-control: private");
	header("Content-type: application/csv");
	header("Content-Disposition: attachment; filename=\"esy_survey_resport_${survey_id}.csv\"");
	header("Pragma: public");

	/*
 	header("Cache-control: private");
	header("Content-disposition: attachment; filename=esy_survey_resport_${survey_id}.csv");
	header("Content-type: application/force-download");
	header("Pragma: public");
	*/

	// send the csv data
	echo $csv_data_str;

	ob_end_flush();
?>
