<?php 
	require_once('./pear/DB.php');
	include_once('./settings/settings.php');
	include_once('./include/globals.php');
	include_once('./include/loader.php');
	include_once('./include/login_functions.php');
	// make sure the user has logged in before accessing this page
	if(($user_struct = get_login_status()) === FALSE){ header("Location: login.php"); exit(0); }

	include_once('./include/functions.php');
	include_once('./include/page_specific/system_user_management.php');

	// place to store all error messages
	$error_log = Array();
	$info_log = Array();

	// make sure all variables that may be passed in get declared
	$page_action = isset($_REQUEST['page_action']) ? $_REQUEST['page_action'] : '';
	$mgt_user_id = isset($_REQUEST['mgt_user_id']) ? $_REQUEST['mgt_user_id'] : '';
	$mgt_user_name = isset($_REQUEST['mgt_user_name']) ? $_REQUEST['mgt_user_name'] : '';
	$mgt_full_name = isset($_REQUEST['mgt_full_name']) ? $_REQUEST['mgt_full_name'] : '';
	$mgt_user_password = isset($_REQUEST['mgt_user_password']) ? $_REQUEST['mgt_user_password'] : '';
	$mgt_user_password2 = isset($_REQUEST['mgt_user_password2']) ? $_REQUEST['mgt_user_password2'] : '';
	$mgt_email_address = isset($_REQUEST['mgt_email_address']) ? $_REQUEST['mgt_email_address'] : '';
	$mgt_user_type_id = isset($_REQUEST['mgt_user_type_id']) ? $_REQUEST['mgt_user_type_id'] : $UID_USER;

	// determine the number of admin users in the system. This is needed for
	// checking if a delete or update is allowed
	$admin_count = get_admin_user_count();

	// basic users always have thier own information loaded
	if($user_struct['user_type_id'] == $UID_USER){
		$mgt_user_type_id = $UID_USER;
		$mgt_user_id = $user_struct['user_id'];
		if($page_action != 'save_user' && $page_action != 'delete_user'){ $page_action = 'load_user'; }
	}

	// perform some state specific actions
	if($page_action == 'save_user'){
		$where = "";
		// if we're doing an update, exclude the record from the search
		if(strlen($mgt_user_id) > 0 && is_numeric($mgt_user_id)){ $where = "user_id <> ${mgt_user_id} AND"; }
		// make sure that the user name being inserted/updated, is unique
		$query = "SELECT COUNT(*) AS matches FROM esy_users WHERE $where user_name = '${mgt_user_name}' AND rec_status_id = 1";
		$db_result = $DBH->query($query);
		if($db_result->numRows() > 0){
			$db_row = $db_result->fetchRow();
			// prevent a save if we're trying to use a user name already in the system 
			if($db_row['matches'] > 0){
				$error_log[] = "This User Name already exists in the system. Please choose another.";
			}
		} // end if

		// if the user is modifying an existing user
		if(strlen($mgt_user_id) > 0 && is_numeric($mgt_user_id)){ 
			$curr_user_struct = get_user_struct_by_user_id($mgt_user_id);
			// make sure that the user isn't trying to modify the last
			// administrator account to be a user. That would result in a system with zero admin accounts (bad)
			if($curr_user_struct['user_type_id'] == $UID_ADMIN && $admin_count == 1 && $mgt_user_type_id == $UID_USER){
				$error_log[] = "You may not change the last remaining Administrator account to a User account.";
			}
		} // end if

		// if no errors were found, attempt to save the user data to the DB.
		if(count($error_log) == 0){
			$mgt_user_id = save_user($mgt_user_id,$mgt_user_type_id,$mgt_user_name,$mgt_user_password,$mgt_full_name,$mgt_email_address);
		}

		// if we're modifying ourselves, make sure our login session data is up
		// to date.
		if($user_struct['user_id'] == $mgt_user_id){
			$user_struct = get_user_struct_by_user_id($mgt_user_id);
			set_login_session($user_struct);
		}
	} // end if
	elseif($page_action == 'delete_user'){
		if(strlen($mgt_user_id) > 0 && is_numeric($mgt_user_id)){
			// users can't delete themselves.
			if($user_struct['user_id'] == $mgt_user_id){
				$error_log[] = "You may not delete yourself from the system.";
			}
			else{
				// see if we're deleting an admin account
				$del_user_struct = get_user_struct_by_user_id($mgt_user_id);
				// if so, make sure it's not the last one in the database
				if($del_user_struct['user_type_id'] == $UID_ADMIN){
					// prevent a delete if we're trying to delete the last admin account
					if($admin_count <= 1){
						$error_log[] = "You may not delete the last remaining Administrator account from the system.";
					}
				} // end if
			} // end else

			// perform the delete if no errors were found
			if(count($error_log) == 0){ 
				$query = "UPDATE esy_users SET rec_status_id = 0 WHERE user_id = ${mgt_user_id} AND rec_status_id = 1";
				$db_result = $DBH->query($query);
				if(DB::isError($db_result)){ die($db_result->message.'<br>'.$db_result->userinfo); }
			} // end if
		} // end if

		// if errors were generated, then simply load the original user data into the form
		if(count($error_log) > 0){ $page_action = 'load_user'; }
	} // end elseif
	if($page_action == 'load_user'){
		$mgt_user_struct = get_user_struct_by_user_id($mgt_user_id);
		if($mgt_user_struct !== FALSE){
			// populate our form variables with this info
			$mgt_user_id = $mgt_user_struct['user_id'];
			$mgt_user_name = $mgt_user_struct['user_name'];
			$mgt_full_name = $mgt_user_struct['full_name'];
			$mgt_email_address = $mgt_user_struct['email_address'];
			$mgt_user_type_id = $mgt_user_struct['user_type_id'];
			$mgt_user_password = $mgt_user_struct['user_password'];
			$mgt_user_password2 = $mgt_user_struct['user_password'];
		} // end if
	} // end if

	// define some template specific variables
	$PAGE_TITLE = "Webupdate | eSurvey 2.0 :: System :: Users";
	$PAGE_HEADER_TEXT = "SYSTEM &#160;<span style='font-size:10px;'>[USER MANAGEMENT]</span>";
	include_once('./include/header.php'); 
	include_once('./include/section_header.php'); 
?>
<!-- MAIN PAGE CONTENT [start] -->				
					<style type='text/css'>
					<!--
						.l_tab_label{
							font: normal normal normal 9px Verdana,Tahoma,Arial,Helvetica,Sans-serif,sans-serif;
							color: #4f5161;
							line-height: 11px;
							text-decoration: none;
						}
						.l_tab_label:hover {
							font: normal normal normal 9px Verdana,Tahoma,Arial,Helvetica,Sans-serif,sans-serif;
							color: #4f5161;
							line-height: 11px;
							text-decoration: underline;
						}
						.l_tab_label_disabled{
							font: normal normal normal 9px Verdana,Tahoma,Arial,Helvetica,Sans-serif,sans-serif;
							color: #9596a0;
							line-height: 11px;
							text-decoration: none;
						}
						.l_user_list_area {
							width: 323px;
							height: 194px;
							background-color: #cfe1e5;
							overflow: scroll;
							overflow-x: hidden;
							overflow-y: scroll;
						}
						.l_user_item {
							width: 263px;
							font: normal normal normal 9px Verdana,Tahoma,Arial,Helvetica,Sans-serif,sans-serif;
							color: #000000;
							line-height: 11px;
							overflow: hidden;
							text-overflow: ellipsis;
						}
						.l_user_link {
							color: #000000;
							text-decoration: none;
						}
						.l_user_link:hover {
							color: #000000;
							text-decoration: underline;
						}
						.l_user_active_link {
							font-weight: bold;
							color: #297181;
							text-decoration: none;
						}
						.l_user_active_link:hover {
							font-weight: bold;
							color: #297181;
							text-decoration: underline;
						}

					-->
					</style>
					<script language='JavaScript' src='./js/form_utils.js'></script>
					<script language='JavaScript' src='./js/page_specific/system_user_management.js'></script>

					<img width='1' height='8' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br>
					<center>
					<form name='system_users_form' id='system_users_form' method='POST' action='<?php echo $_SERVER['PHP_SELF']; ?>'>

					<table cellspacing='0' cellpadding='0' border='0'>
					<tr>
						<td width='696' align='center' valign='top' bgcolor='#dcdcdf'>
							<img width='1' height='8' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br>
							<table cellspacing='0' cellpadding='0' border='0'>
							<tr>
								<td colspan='3' width='676' height='30' align='left' valign='bottom' background='./images/system/other/other_tab_area_bg.gif'>
									<table cellspacing='0' cellpadding='0' border='0'>
									<tr>
										<td width='146' height='30' align='center' valign='middle' bgcolor='#cfe1e5' background='./images/system/other/other_tab_bg_on.gif'><span class='l_tab_label'>USER MANAGEMENT</span><br></td>
										<td width='4' height='30'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
<?php if($user_struct['user_type_id'] == $UID_ADMIN){ ?>
										<td width='146' height='30' align='center' valign='middle' bgcolor='#91b6be' background='./images/system/other/other_tab_bg_off.gif'><a href='system_options.php' class='l_tab_label'>SYSTEM OPTIONS</a><br></td>
										<td width='4' height='30'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
										<td width='146' height='30' align='center' valign='middle' bgcolor='#91b6be' background='./images/system/other/other_tab_bg_off.gif'><a href='system_pub_points.php' class='l_tab_label'>PUBLISH POINTS</a><br></td>
<?php }else{ ?>										
										<td width='146' height='30' align='center' valign='middle' bgcolor='#b6c9ce' background='./images/system/other/other_tab_bg_null.gif'><span class='l_tab_label_disabled'>SYSTEM OPTIONS</span><br></td>
										<td width='4' height='30'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
										<td width='146' height='30' align='center' valign='middle' bgcolor='#b6c9ce' background='./images/system/other/other_tab_bg_null.gif'><span class='l_tab_label_disabled'>PUBLISH POINTS</span><br></td>
<?php } ?>
									</tr>
									</table>
								</td>
							</tr>
							<tr>
								<td width='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
								<td width='674' align='center' valign='top' bgcolor='#cfe1e5'>
									<img width='1' height='8' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br>
									<table cellspacing='0' cellpadding='0' border='0'>
									<tr>
										<td width='328' align='left' valign='top'>
<!-- USER DETAILS [start] -->
											<table cellspacing='0' cellpadding='0' border='0'>
											<tr>
												<td width='328' colspan='3' height='20' align='left' valign='middle' bgcolor='#4f5161'>
													<table cellspacing='0' cellpadding='0' border='0'>
													<tr>
														<td width='239'>
															<table cellspacing='0' cellpadding='0' border='0'>
															<tr>
																<td><span class='g_sub_section_header'><b>User Details</b></span></td>
																<td><a href='JavaScript:void(0);' onclick='popup_context_help("./help/user_management.html#UserDetails");'><img onmouseover='this.src=icon_context_help_on.src;' onmouseout='this.src=icon_context_help_off.src;' width='13' height='13' src='./images/global/icon/icon_context_help_off.gif' alt='Help' title='Help' hspace='0' vspace='0' border='0'></a><br></td>
															</tr>
															</table>
														</td>
														<td width='89' align='right'>
															<?php 
															// hide the create new user button if we're a basic user
															if($user_struct['user_type_id'] == $UID_ADMIN){ ?>
															<a href='system_user_management.php'><img onmouseover='this.src=bttn_create_new_user_on.src;' onmouseout='this.src=bttn_create_new_user_off.src;' align='right' width='89' height='19' src='./images/system/button/bttn_create_new_user_off.gif' alt='Create New User' title='Create New User' hspace='0' vspace='0' border='0'></a>
															<?php } ?>
														</td>
													</tr>
													</table>
												</td>
											</tr>
											<tr>
												<td width='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
												<td width='326' height='194' align='left' valign='top' bgcolor='#ffffff' background='./images/system/other/other_pp_form_bg.gif'>
													<img width='1' height='3' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br>
													<input type='hidden' id='mgt_user_id' name='mgt_user_id' value='<?php echo to_html_attrib_str($mgt_user_id); ?>'>
													<input type='hidden' id='page_action' name='page_action' value=''>
													<table cellspacing='0' cellpadding='0' border='0'>
													<tr>
														<td width='18' height='23' align='center' valign='middle'><img id='mgt_user_name_err_img' width='14' height='14' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
														<td id='mgt_user_name_label' width='100' height='23' align='right' valign='middle' class='g_form_field_label'>User Name: &#160;<br></td>
														<td width='208' height='23' align='left' valign='middle'><input type='text' id='mgt_user_name' name='mgt_user_name' value='<?php echo to_html_attrib_str($mgt_user_name); ?>' maxlength='32' class='g_form_text_input' style='width:196px; margin-left:5px;'></td>
													</tr>
													<tr>
														<td width='18' height='23' align='center' valign='middle'><img id='mgt_full_name_err_img' width='14' height='14' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
														<td id='mgt_full_name_label' width='100' height='23' align='right' valign='middle' class='g_form_field_label'>Full Name: &#160;<br></td>
														<td width='208' height='23' align='left' valign='middle'><input type='text' id='mgt_full_name' name='mgt_full_name' value='<?php echo to_html_attrib_str($mgt_full_name); ?>' maxlength='256' class='g_form_text_input' style='width:196px; margin-left:5px;'></td>
													</tr>
													<tr>
														<td width='18' height='23' align='center' valign='middle'><img id='mgt_email_address_err_img' width='14' height='14' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
														<td id='mgt_email_address_label' width='100' height='23' align='right' valign='middle' class='g_form_field_label'>Email Address: &#160;<br></td>
														<td width='208' height='23' align='left' valign='middle'><input type='text' id='mgt_email_address' name='mgt_email_address' value='<?php echo to_html_attrib_str($mgt_email_address); ?>' maxlength='256' class='g_form_text_input' style='width:196px; margin-left:5px;'></td>
													</tr>
													<tr>
														<td width='18' height='23' align='center' valign='middle'><img id='mgt_user_type_id_err_img' width='14' height='14' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
														<td id='mgt_user_type_id_label' width='100' height='23' align='right' valign='middle' class='g_form_field_label'>User Type: &#160;<br></td>
														<td width='208' height='23' align='left' valign='middle'>
															<?php
																// the user type id is disabled for users so they can't make themselves into admins
																$disabled = "";
																if($user_struct['user_type_id'] == $UID_USER){ $disabled = "disabled"; }
																echo "<select $disabled id='mgt_user_type_id' name='mgt_user_type_id' size='1' class='g_form_select' style='width:196px; margin-left:5px;'>\n";

																$query = 'SELECT user_type_id, user_type_label FROM esy_user_types WHERE rec_status_id = 1;';
																$db_result = $DBH->query($query);
																if(!DB::isError($db_result)){ 
																	while($db_row = $db_result->fetchRow()){ 
																		$selected = ($mgt_user_type_id == $db_row['user_type_id']) ? "selected" : "";
																		echo "<option class='g_form_select_option' value='".$db_row['user_type_id']."' $selected>".$db_row['user_type_label']."</option>\n";
																	}
																}
																echo "</select>\n";
															?>
														</td>
													</tr>
													<tr>
														<td width='18' height='23' align='center' valign='middle'><img id='mgt_user_password_err_img' width='14' height='14' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
														<td id='mgt_user_password_label' width='100' height='23' align='right' valign='middle' class='g_form_field_label'>Password: &#160;<br></td>
														<td width='208' height='23' align='left' valign='middle'><input type='password' id='mgt_user_password' name='mgt_user_password' value='<?php echo to_html_attrib_str($mgt_user_password); ?>' maxlength='32' class='g_form_text_input' style='width:196px; margin-left:5px;'></td>
													</tr>
													<tr>
														<td width='18' height='23' align='center' valign='middle'><img id='mgt_user_password2_err_img' width='14' height='14' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
														<td id='mgt_user_password2_label' width='100' height='23' align='right' valign='middle' class='g_form_field_label'>Retype Password: &#160;<br></td>
														<td width='208' height='23' align='left' valign='middle'><input type='password' id='mgt_user_password2' name='mgt_user_password2' value='<?php echo to_html_attrib_str($mgt_user_password2); ?>' maxlength='32' class='g_form_text_input' style='width:196px; margin-left:5px;'></td>
													</tr>
													</table>
												</td>
												<td width='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
											</tr>
											<tr>
												<td colspan='3' width='328' height='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
											</tr>
											</table>
<!-- USER DETAILS [end] -->
										</td>
										<td width='8'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
										<td width='325' align='left' valign='top'>
<!-- USER LIST [start] -->
											<table cellspacing='0' cellpadding='0' border='0'>
											<tr>
												<td width='325' colspan='3' height='20' align='left' valign='middle' bgcolor='#4f5161'>
													<table cellspacing='0' cellpadding='0' border='0'>
													<tr>
														<td><span class='g_sub_section_header'><b>User List</b></span></td>
														<td><a href='JavaScript:void(0);' onclick='popup_context_help("./help/user_management.html#UserList");'><img onmouseover='this.src=icon_context_help_on.src;' onmouseout='this.src=icon_context_help_off.src;' width='13' height='13' src='./images/global/icon/icon_context_help_off.gif' alt='Help' title='Help' hspace='0' vspace='0' border='0'></a><br></td>
													</tr>
													</table>
												</td>
											</tr>
											<tr>
												<td width='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
												<td width='323' align='left' valign='top' bgcolor='#ffffff'>
													<div class='l_user_list_area'>
														<?php echo get_user_list_html($user_struct['user_id'], $mgt_user_id); ?>
													</div>
												</td>
												<td width='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
											</tr>
											<tr>
												<td colspan='3' width='328' height='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
											</tr>
											</table>
<!-- USER LIST [end] -->
										</td>
									</tr>
									</table>
									<img width='1' height='8' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br>
								</td>
								<td width='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
							</tr>
							<tr>
								<td colspan='3' width='676' height='1' bgcolor='#4f5161'><img width='1' height='1' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br></td>
							</tr>
							</table>
							<img width='1' height='8' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br>
						</td>
					</tr>
					<tr>
						<td height='30' align='right' valign='bottom'>
							<a align='right' href='survey_list.php'><img width='73' height='19' onmouseover='this.src=bttn_cancel_on.src;' onmouseout='this.src=bttn_cancel_off.src;' src='./images/global/button/bttn_cancel_off.gif' alt='Cancel' title='Cancel' hspace='2' vspace='0' border='0'></a>
							<a align='right' href='JavaScript:void(0);' onmouseup='save_user_onmouseup();'><img width='73' height='19' onmouseover='this.src=bttn_save_on.src;' onmouseout='this.src=bttn_save_off.src;' src='./images/global/button/bttn_save_off.gif' alt='Save' title='Save' hspace='0' vspace='0' border='0'></a><br>
						</td>
					</tr>
					</table>

					</form>
					</center>
					<img width='1' height='5' src='./images/global/other/other_clear_spacer.gif' hspace='0' vspace='0' border='0'><br>

					<script language='JavaScript'>
					<!--
<?php
	// if the user was succesfully written, alert the user
	/*
	if($page_action == 'save_user' && count($error_log) == 0){
		echo "alert('[ SUCCESS ]\\nThe User has been successfully saved to the database.');\n";
	}
	*/

	// if errors were found, format them for display in a single alert box
	if(count($error_log) > 0){ 
		$error_alert = "[ ERROR ]\n";
		foreach($error_log as $idx => $error_msg){
			$error_alert .= "* $error_msg\n";
		}
		echo "alert('".to_js_str($error_alert)."');\n";
	}
?>

					//-->
					</script>
<!-- MAIN PAGE CONTENT [end] -->				
<?php include_once('./include/footer.php'); ?>
